import base64
import json

from flask import request, jsonify, current_app
from flask_restx import Resource, Namespace, reqparse
from sqlalchemy import select, insert, delete, update, func
from sqlalchemy.orm import Session
from werkzeug.datastructures import FileStorage

from app.defines import StatesCode, Module, OperationType
from app.modle.organization import Company, Department
from app.utils.jwt_util import login_required
from app.utils.save_log import save_log
from app.utils.util import to_dict
from config import Config

ns = Namespace('organization', description='组织管理接口')

company_list = reqparse.RequestParser(bundle_errors=True)
company_list.add_argument(name='page_size', type=int, location='args', required=False, help='每页记录数量，默认:20')
company_list.add_argument(name='page', type=int, location='args', required=False, help='页数')
company_list.add_argument(name='name', type=str, location='args', required=False, help='过滤公司名')

config = Config()


@ns.route('/company_list')
class CompanyListApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='get_company_list', description='获取公司列表')
    @ns.expect(company_list)
    def get(self):
        """获取公司列表"""

        page_size = int(request.args.get('page_size', 20))
        page = int(request.args.get('page', 1))
        company_name = request.args.get('name')

        try:
            with Session(current_app.engine) as session:
                count = select(func.count(Company.id))
                count_results = session.execute(count).scalars().first()

                stmt = select(Company)
                if company_name:
                    stmt = stmt.where(Company.company_name.like("%{}%".format(company_name)))

                stmt = stmt.offset(page_size * (page - 1)).limit(page_size)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', total=count_results, data=to_dict(results))

        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


company_details = reqparse.RequestParser(bundle_errors=True)
company_details.add_argument(name='company_id', type=int, location='form', required=False, help='公司id')
company_details.add_argument(name='company_name', type=str, location='form', required=True, help='公司名')
company_details.add_argument(name='logo', type=FileStorage, location='files', required=False, help='商标')
company_details.add_argument(name='abbreviation', type=str, location='form', required=False, help='公司简称')
company_details.add_argument(name='code', type=str, location='form', required=False, help='统一社会信用代码')
company_details.add_argument(name='registered_address', type=str, location='form', required=False, help='注册地址')
company_details.add_argument(name='business_address', type=str, location='form', required=False, help='经营地址')
company_details.add_argument(name='legal_person_name', type=str, location='form', required=False, help='法人姓名')
company_details.add_argument(name='company_code', type=str, location='form', required=False, help='公司编码')
company_details.add_argument(name='staff_size', type=str, location='form', required=False, help='编制人数')
company_details.add_argument(name='on_guard_size', type=str, location='form', required=False, help='在岗人数')

delete_company = reqparse.RequestParser(bundle_errors=True)
delete_company.add_argument(name='company_id', type=int, location='form', required=False, help='公司id')

get_company_details = reqparse.RequestParser(bundle_errors=True)
get_company_details.add_argument(name='company_id', type=int, location='args', required=False, help='公司id')


@ns.route('/company')
class CompanyApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='get_company_details', description='获取公司详情')
    @ns.expect(get_company_details)
    def get(self):
        """获取公司详情"""

        company_id = request.args.get('company_id')

        if company_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="公司id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = select(Company).where(Company.id == company_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=to_dict(results))
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_company', description='添加公司')
    @ns.expect(company_details)
    def post(self):
        """添加公司"""

        company_name = request.form.get('company_name')
        logo = request.files.get('logo')
        abbreviation = request.form.get('abbreviation')
        code = request.form.get('code')
        registered_address = request.form.get('registered_address')
        business_address = request.form.get('business_address')
        legal_person_name = request.form.get('legal_person_name')
        company_code = request.form.get('company_code')
        staff_size = request.form.get('staff_size')
        on_guard_size = request.form.get('on_guard_size')

        if company_name is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="公司名不能为空")

        if logo:
            ext = logo.filename.split('.')[-1]
            base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(logo.read())
            base64_data = base64_data.decode('utf-8')
        else:
            base64_data = None

        try:
            with Session(current_app.engine) as session:
                # 判断公司是否存在
                stmt = select(Company).where(Company.company_name == company_name)
                result = session.execute(stmt).scalars().first()
                if result:
                    return jsonify(code=StatesCode.UNKNOWN_ERROR, message="公司已存在")

                # 添加公司
                stmt = insert(Company).values(
                    company_name=company_name,
                    logo=base64_data,
                    abbreviation=abbreviation,
                    code=code,
                    registered_address=registered_address,
                    business_address=business_address,
                    legal_person_name=legal_person_name,
                    company_code=company_code,
                    staff_size=staff_size,
                    on_guard_size=on_guard_size
                )
                session.execute(stmt)

                session.commit()

            save_log(request, Module.ORGANIZATION, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='添加成功')
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.ADD, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_company_details', description='修改公司详情')
    @ns.expect(company_details)
    def put(self):
        """更新公司详情"""
        company_id = request.form.get('company_id')
        company_name = request.form.get('company_name')
        logo = request.files.get('logo')
        abbreviation = request.form.get('abbreviation')
        code = request.form.get('code')
        registered_address = request.form.get('registered_address')
        business_address = request.form.get('business_address')
        legal_person_name = request.form.get('legal_person_name')
        company_code = request.form.get('company_code')
        staff_size = request.form.get('staff_size')
        on_guard_size = request.form.get('on_guard_size')

        if company_name is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="公司名不能为空")

        if logo:
            ext = logo.filename.split('.')[-1]
            base64_data = b'data:image/%s;base64,' % ext.encode('utf-8') + base64.b64encode(logo.read())
            base64_data = base64_data.decode('utf-8')
        else:
            base64_data = None

        try:
            with Session(current_app.engine) as session:
                stmt = update(Company).where(Company.id == company_id).values(
                    company_name=company_name,
                    logo=base64_data,
                    abbreviation=abbreviation,
                    code=code,
                    registered_address=registered_address,
                    business_address=business_address,
                    legal_person_name=legal_person_name,
                    company_code=company_code,
                    staff_size=staff_size,
                    on_guard_size=on_guard_size
                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.ORGANIZATION, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='修改成功')
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.UPDATE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='delete_company', description='删除公司')
    @ns.expect(company_details)
    def delete(self):
        """删除公司"""
        company_id = request.form.get('company_id')

        if company_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="公司id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = delete(Company).where(Company.id == company_id)

                session.execute(stmt)
                session.commit()

            save_log(request, Module.ORGANIZATION, OperationType.DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message="删除成功")
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


department_details = reqparse.RequestParser(bundle_errors=True)
department_details.add_argument(name='department_id', type=str, location='form', required=False, help='部门id')
department_details.add_argument(name='company_id', type=str, location='form', required=False, help='上级公司')
department_details.add_argument(name='department_name', type=str, location='form', required=False, help='部门名')
department_details.add_argument(name='principal', type=str, location='form', required=False, help='负责人')
department_details.add_argument(name='liaison_officer', type=str, location='form', required=False, help='接口人')
department_details.add_argument(name='duplicate_name', type=str, location='form', required=False, help='拍重名')
department_details.add_argument(name='on_guard', type=str, location='form', required=False, help='在岗人数')
department_details.add_argument(name='freelance_staff', type=str, location='form', required=False, help='自由员工')
department_details.add_argument(name='cooperation_staff', type=str, location='form', required=False, help='协作员工')
department_details.add_argument(name='thirdparty_staff', type=str, location='form', required=False, help='第三方员工')
department_details.add_argument(name='censure', type=str, location='form', required=False, help='职责')


@ns.route('/department_list')
class DepartmentListApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='department_list', description='获取公司下部门列表')
    @ns.expect(get_company_details)
    def get(self):
        """获取公司下部门列表"""

        company_id = request.args.get('company_id')

        if company_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="公司id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = select(Department.id, Department.department_name).where(Department.company_id == company_id)
                results = session.execute(stmt)

            data = []
            for result in results:
                data.append({"id": result.id, "department_name": result.department_name})

            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功', data=data)
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


get_department = reqparse.RequestParser(bundle_errors=True)
get_department.add_argument(name='department_id', type=str, location='args', required=False, help='部门id')


@ns.route('/department')
class DepartmentApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='get_department_details', description='获取部门详情')
    @ns.expect(get_department)
    def get(self):
        """获取部门详情"""

        department_id = request.args.get('department_id')

        if department_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="部门id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = select(Department).where(Department.id == department_id)
                results = session.execute(stmt).scalars().all()

            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message="成功", data=to_dict(results))
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='add_department', description='添加部门')
    @ns.expect(department_details)
    def post(self):
        """添加部门"""
        company_id = request.form.get('company_id')
        department_name = request.form.get('department_name')
        principal = request.form.get('principal')
        liaison_officer = request.form.get('liaison_officer')
        duplicate_name = request.form.get('duplicate_name')
        on_guard = request.form.get('on_guard')
        freelance_staff = request.form.get('freelance_staff')
        cooperation_staff = request.form.get('cooperation_staff')
        thirdparty_staff = request.form.get('thirdparty_staff')
        censure = request.form.get('censure')

        if company_id is None or department_name is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="上级公司和部门名称不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = insert(Department).values(
                    company_id=company_id,
                    department_name=department_name,
                    principal=principal,
                    liaison_officer=liaison_officer,
                    duplicate_name=duplicate_name,
                    on_guard=on_guard,
                    freelance_staff=freelance_staff,
                    cooperation_staff=cooperation_staff,
                    thirdparty_staff=thirdparty_staff,
                    censure=censure,

                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.ORGANIZATION, OperationType.ADD, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.INQUIRE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='update_company_details', description='修改部门详情')
    @ns.expect(department_details)
    def put(self):
        """更新部门详情"""

        department_id = request.form.get('department_id')
        company_id = request.form.get('company_id')
        department_name = request.form.get('department_name')
        principal = request.form.get('principal')
        liaison_officer = request.form.get('liaison_officer')
        duplicate_name = request.form.get('duplicate_name')
        on_guard = request.form.get('on_guard')
        freelance_staff = request.form.get('freelance_staff')
        cooperation_staff = request.form.get('cooperation_staff')
        thirdparty_staff = request.form.get('thirdparty_staff')
        censure = request.form.get('censure')

        if company_id is None or department_name is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="上级公司和部门名称不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = update(Department).where(Department.id == department_id).values(
                    company_id=company_id,
                    department_name=department_name,
                    principal=principal,
                    liaison_officer=liaison_officer,
                    duplicate_name=duplicate_name,
                    on_guard=on_guard,
                    freelance_staff=freelance_staff,
                    cooperation_staff=cooperation_staff,
                    thirdparty_staff=thirdparty_staff,
                    censure=censure,

                )
                session.execute(stmt)
                session.commit()

            save_log(request, Module.ORGANIZATION, OperationType.UPDATE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message="成功")
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.UPDATE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))

    @ns.doc(id='delete_company_details', description='删除部门')
    @ns.expect(department_details)
    def delete(self):
        """删除部门"""
        department_id = request.form.get('department_id')

        if department_id is None:
            return jsonify(code=StatesCode.PARA_ERROR, message="部门id不能为空")

        try:
            with Session(current_app.engine) as session:
                stmt = delete(Department).where(Department.id == department_id)
                session.execute(stmt)
                session.commit()

            save_log(request, Module.ORGANIZATION, OperationType.DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='成功')
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))


batch_delete_company = reqparse.RequestParser(bundle_errors=True)
batch_delete_company.add_argument(name='company_ids', type=list, location='form', required=False, help='公司ids')


@ns.route('/batch_delete_company')
class BatchDeleteCompanyApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='batch_delete_users', description='批量删除公司')
    @ns.expect(batch_delete_company)
    def delete(self):
        """批量删除公司"""
        company_ids = request.form.get('company_ids')

        if company_ids is not None:
            company_ids = json.loads(company_ids)
        else:
            return jsonify(code=StatesCode.PARA_ERROR, message='公司id不能为空')

        try:
            with Session(current_app.engine) as session:
                stmt = select(Company).where(Company.id.in_(company_ids))
                results = session.execute(stmt).scalars().all()

                for result in results:
                    session.delete(result)
                session.commit()

            save_log(request, Module.ORGANIZATION, OperationType.BATCH_DELETE, StatesCode.SUCCESS)

            return jsonify(code=StatesCode.SUCCESS, message='批量删除成功')
        except Exception as e:
            save_log(request, Module.ORGANIZATION, OperationType.BATCH_DELETE, StatesCode.UNKNOWN_ERROR)
            return jsonify(code=StatesCode.UNKNOWN_ERROR, message=str(e))
